# -*- coding: utf-8 -*-
#  Copyright (C) 2006, 2007 Stewart Adam
#  This file is part of fwbackups.

#  fwbackups is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.

#  fwbackups is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with fwbackups; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
Graphical utilities for fwbackups
"""

import gtk
import time
import fwbackups
from const import *
from i18n import _
import widgets

def busyCursor(mainwin,insensitive=False):
  """ Set busy cursor in mainwin and make it insensitive if selected """
  mainwin.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
  if insensitive:
    mainwin.set_sensitive(False)
  doGtkEvents()

def normalCursor(mainwin):
  """ Set Normal cursor in mainwin and make it sensitive """
  mainwin.window.set_cursor(None)
  mainwin.set_sensitive(True)    
  doGtkEvents()

def doGtkEvents():
  """ Process gtk events """
  while gtk.events_pending():
    gtk.main_iteration()

def getSelectedItems(widget):
  """ Get selected items from icon/tree view """
  try:
    model = widget.get_model()
    selected = widget.get_selected_items()
    return selected
  except:
    return False

def runFuncAsThread(functorun):
  """ Runs a function as a new thread """
  thread = fwbackups.FuncAsThread(functorun)
  thread.start()
  return thread

def runClassAsThread(classtorun, *kargs):
  """ Runs a function as a new thread """
  thread = fwbackups.ClassAsThread(classtorun, kargs)
  thread.start()
  return thread


def displayInfo(self, parent, headerMessage, message, dontShowMe=False):
  """ Wrapper for displaying the confirm dialog """
  if dontShowMe:
    dontShowMe = self.ui.infoDiaDontShowMeCheck
  dialog = widgets.InfoDia(self.ui.info_dia, _('Information'), parent,
                              self.ui.infoDiaHeaderLabel,
                              self.ui.infoDiaMessageLabel,
                              headerMessage, message, dontShowMe)
  dialog.run()
  if dontShowMe:
    showMe = dontShowMe.get_active()
    dialog.destroy()
    return showMe
  else:
    dialog.destroy()
    return
  

def displayError(self, parent, headerMessage, message):
  """ Wrapper for displaying the confirm dialog """
  dialog = widgets.ErrorDia(self.ui.error_dia, _('Error'), parent,
                              self.ui.errorDiaHeaderLabel,
                              self.ui.errorDiaMessageLabel,
                              headerMessage, message)
  dialog.run()
  dialog.destroy()

def displayWarning(self, parent, headerMessage, message, dontShowMe=False):
  """ Wrapper for displaying the confirm dialog """
  if dontShowMe:
    dontShowMe = self.ui.warningDiaDontShowMeCheck
  dialog = widgets.WarningDia(self.ui.warning_dia, _('Warning'), parent,
                              self.ui.warningDiaHeaderLabel,
                              self.ui.warningDiaMessageLabel,
                              headerMessage, message, dontShowMe)
  dialog.run()
  if dontShowMe:
    showMe = dontShowMe.get_active()
    dialog.destroy()
    return showMe
  else:
    dialog.destroy()
    return
  
def displayConfirm(self, parent, headerMessage, message, dontShowMe=False):
  """ Wrapper for displaying the confirm dialog """
  if dontShowMe:
    dontShowMe = self.ui.confirmDiaDontShowMeCheck
  dialog = widgets.ConfirmDia(self.ui.confirm_dia, _('Please Confirm'), parent,
                              self.ui.confirmDiaHeaderLabel,
                              self.ui.confirmDiaMessageLabel,
                              headerMessage, message, dontShowMe)
  response = dialog.run()
  if dontShowMe:
    showMe = dontShowMe.get_active()
    dialog.destroy()
    return response, showMe
  else:
    dialog.destroy()
    return response
  
