# -*- coding: utf-8 -*-
#  Copyright (C) 2007, 2008 Stewart Adam
#  This file is part of fwbackups.

#  fwbackups is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.

#  fwbackups is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with fwbackups; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
Functions for operations on remote computers
"""
import paramiko
import socket
import sys
import time

from fwbackups.const import *
from fwbackups.i18n import _

def receiveFile(host, username, password, port, src, dst):
  """
  Opens a SFTP connections and gets src (remote) to
  dest (local).
  """
  #paramiko.util.log_to_file('%s/paramiko_client.log' % LOC)
  port = int(port)
  client = paramiko.SSHClient()
  client.load_system_host_keys()
  client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
  client.connect(host, port, username, password, timeout=10)
  # open sftp, upload then close
  sftp = client.open_sftp()
  sftp.get(src, dst)
  sftp.close()
  client.close()


def transferFile(host, username, password, port, src, dst, symlinks=False):
  """
  Opens a SFTP connections and moves src (local) to
  dest (remote). Assumes dst is a folder and already exists.
  """
  #paramiko.util.log_to_file('%s/paramiko_client.log' % LOC)
  port = int(port)
  client = paramiko.SSHClient()
  client.load_system_host_keys()
  client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
  client.connect(host, port, username, password, timeout=10)
  # open sftp, upload then close
  sftp = client.open_sftp()
  sftp.put(src, dst)
  sftp.close()
  client.close()

def transferFolder(host, username, password, port, src, dst, symlinks=False):
  """
  Opens a SFTP connections and moves src (local) to
  dest (remote). Assumes dst is a folder and already exists.
  """
  #paramiko.util.log_to_file('%s/paramiko_client.log' % LOC)
  port = int(port)
  client = paramiko.SSHClient()
  try:
    transport = client.get_transport()
    transport.use_compression(True)
  except:
    # 1.7.2 isn't installed
    pass
  client.load_system_host_keys()
  client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
  client.connect(host, port, username, password, timeout=10)
  # open sftp, upload then close
  sftp = client.open_sftp()
  errors = []
  try:
    print 'chdir ' + os.path.dirname(dst)
    #sftp.chdir(dst)
    for dirpath, dirnames, filenames in os.walk(src):
      fullpath = os.path.join(src, dirpath)
      #print 'mkdir ' + ConvertPath('%s/%s' % (dst, dirpath.replace(src, '')))
      try:
        sftp.mkdir(ConvertPath('%s/%s' % (dst, dirpath.replace(src, ''))))
        test = 'test'
      except IOError:
        #print 'IOError! Maybe it exists already?'
        pass
      #print 'chdir ', ConvertPath('%s/%s' % (dst, dirpath.replace(src, '')))
      sftp.chdir(ConvertPath('%s/%s' % (dst, dirpath.replace(src, ''))))
      for filename in filenames:
        fullfilename = os.path.join(fullpath, filename)
        try:
          if symlinks and os.path.islink(fullfilename):
            linkto = os.readlink(fullfilename)
            #dbg print 'linking %s to %s' % (linkto, filename)
            sftp.symlink(linkto, filename)
          elif not os.path.isfile(fullfilename):
            print _('`%s\' isn\'t a file, folder or link! Skipping.') % fullfilename
          else: # file
            #dbg print 'putting %s to %s' % (fullfilename, filename)
            sftp.put(fullfilename, os.path.split(filename)[1])
            pass
        except (IOError, os.error), reason:
          errors.append('%s --> %s: %s' % (fullfilename, os.path.split(filename)[1], reason))
      if errors:
        print _('Couldn\'t copy some files due to errors:')
        print ''.join(errors)
        pass
      #dbg print '***'
  except:
    sftp.close()
    client.close()
    raise  
  sftp.close()
  client.close()

# Remember we wrap this in callbacks.py
def testConnection(host, username, password, port, path):
  """
  Tests a SSH/SFTP connection with the supplied arguments
  Return True or False depending on the connection results
  """
  port = int(port)
  #paramiko.util.log_to_file('%s/paramiko_client.log' % LOC)
  client = paramiko.SSHClient()
  client.load_system_host_keys()
  client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
  client.connect(host, port, username, password, timeout=30)
  sftp = client.open_sftp()
  try:
    sftp.stat(path)
    exists = True
  except IOError, errorDesc:
    # no such file.
    exists = False
  sftp.close()
  client.close()
  return exists


